-------------------------------------------------------------------------------
-- Compass Distance 0.3
-------------------------------------------------------------------------------
--
-- Copyright (c) 2014 Ales Machat (Garkin)
-- 
-- Permission is hereby granted, free of charge, to any person
-- obtaining a copy of this software and associated documentation
-- files (the "Software"), to deal in the Software without
-- restriction, including without limitation the rights to use,
-- copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the
-- Software is furnished to do so, subject to the following
-- conditions:
-- 
-- The above copyright notice and this permission notice shall be
-- included in all copies or substantial portions of the Software.
-- 
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-- EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
-- OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
-- NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
-- HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-- FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
-- OTHER DEALINGS IN THE SOFTWARE.
--
-------------------------------------------------------------------------------
local CompassDistance = ZO_Object:Subclass()

function CompassDistance:New()
   local obj = ZO_Object.New(self)
   obj:Initialize()
   return obj
end

function CompassDistance:Initialize()
   self.control = WINDOW_MANAGER:CreateTopLevelWindow()
   self.control:SetAnchor(LEFT, COMPASS.control, RIGHT, 25, 0)
   self.control:SetHidden(true)

   self.label = WINDOW_MANAGER:CreateControl(nil, self.control, CT_LABEL)
   self.label:SetAnchor(LEFT, self.control, LEFT, 0, 0)
   self.label:SetFont("ZoFontWinH3")
   self.label:SetColor(ZO_CONTRAST_TEXT:UnpackRGBA())
   
   self.defaultScales = { [0] = 252.5, [1] = 550, [2] = 3200, [3] = 25250, [4] = 0, [5] = 0 }  
   self.mapScales = {
      [2]   = { 799.194412, 3782.72053 }, --Glenumbra      4963.76672 (1.3122214)
      [4]   = { 416.092805, 3204.40927 }, --Stormhaven     4624.15008 (1.4430585)
      [5]   = { 481.867332, 3065.04015 }, --Rivenspire     4062.45840 (1.3254177)
      [9]   = { 659.913551, 3145.94016 }, --Stonefalls     4770.41664 (1.5163723)
      [10]  = { 719.320961, 3222.36718 }, --Deshaan        5347.72480 (1.6595641)
      [11]  = { 679.578661, 3246.85866 }, --Malabal Tor    4326.33344 (1.3324674)
      [14]  = { 709.300346, 3355.70470 }, --Bankorai       4344.62512 (1.2946983)
      [15]  = { 563.497743, 4028.19738 }, --Eastmarch      5350.43344 (1.3282451)
      [16]  = { 565.377418, 4021.55554 }, --The Rift       5350.43344 (1.3304388)
      [17]  = { 544.489413, 3133.46034 }, --Alik'r Desert  4680.32928 (1.4936616)
      [18]  = { 439.641604, 3018.32121 }, --Greenshade     3916.12496 (1.2974514)
      [19]  = { 612.906587, 3008.78565 }, --Shadowfen      4012.83344 (1.3337053)
      [37]  = { [0] = 75.5, 252, 4525.73943 }, --Cyrodiil  6019.23344 (1.33)
      [154] = { 479.966871, 4271.31386 }, --Coldharbour    6000.00000 (1.4047200)
      [178] = { 656.525536, 3628.84204 }, --Auridon        5350.46688 (1.4744281)
      [179] = { 611.327549, 3350.53273 }, --Reaper's March 4439.96256 (1.3251512)
      [180] = { 715.000715, 3491.13252 }, --Grahtwood      4624.15008 (1.3245416)
      [352] = { [2] = 3520 },             --Craglorn       4681,60000 (1.33)
   }
   for k,v in pairs(self.mapScales) do
      setmetatable(self.mapScales[k], { __index = self.defaultScales })
   end
   setmetatable(self.mapScales, { __index = function() return self.defaultScales end })

   self.zoneIDs = {   
      ["alikr"]        = 18,
      ["auridon"]      = 179,
      ["bangkorai"]    = 15,
      ["coldharbor"]   = 155,
      ["craglorn"]     = 353,
      ["cyrodiil"]     = 38,
      ["deshaan"]      = 11, 
      ["eastmarch"]    = 16,
      ["glenumbra"]    = 2,
      ["grahtwood"]    = 181,
      ["greenshade"]   = 19,
      ["malabaltor"]   = 12,
      ["reapersmarch"] = 180,
      ["rivenspire"]   = 5,
      ["shadowfen"]    = 20,
      ["stonefalls"]   = 9,
      ["stormhaven"]   = 4,
      ["therift"]      = 17, 
   }
   setmetatable(self.zoneIDs, { __index = function() return 0 end }) 
   
   self:UpdateScale()
   
   CALLBACK_MANAGER:RegisterCallback("CustomCompassPins_MapChanged", function() self:UpdateScale() end)
end

function CompassDistance:SetText(text)
   self.label:SetText(text)
end

function CompassDistance:UpdateScale()
   if ZGV and ZGV.MapData then
      local subzone = select(3,(GetMapTileTexture()):lower():find("maps/[%w%-]+/([%w%-]+_%w+)"))
      local subzoneData = ZGV.MapData.Zones[subzone]
   
      if subzoneData then 
         local worldsize = 33440 --Tamriel size in meters
         if subzoneData.parentWorld == "coldharbour_base" then
            worldsize = 6000     --Coldharbour size in meters
         end
         local scale = subzoneData.scale or 0.01
      
         self.scale = scale * worldsize
         
         return
      end
   end

   local mapType = GetMapType()
   local zoneId = GetCurrentMapZoneIndex() --GetCurrentPlayerZoneIndex()
   if zoneId == nil then
       local zone = select(3,(GetMapTileTexture()):lower():find("maps/([%w%-]+)/"))
       zoneId = self.zoneIDs[zone]
   end

   self.scale = self.mapScales[zoneId][mapType]
end

function CompassDistance:GetScale()
   return self.scale or 0
end

EVENT_MANAGER:RegisterForEvent("CompassDistance", EVENT_ADD_ON_LOADED,
   function(event, addonName)
      if addonName ~= "CompassDistance" then return end

      local COMPASS_DISTANCE = CompassDistance:New()
      HUD_SCENE:AddFragment(ZO_SimpleSceneFragment:New(COMPASS_DISTANCE.control))

      ZO_PreHookHandler(COMPASS.container, "OnUpdate",
         function()
            if not COMPASS.centerOverPinLabelAnimation:IsPlaying() or not COMPASS.centerOverPinLabelAnimation:IsPlayingBackward() then
               local now = GetFrameTimeMilliseconds()
               if now < COMPASS.nextLabelUpdateTime then
                  return true
               end

               local newDistance
               if not (DoesUnitExist("boss1") or DoesUnitExist("boss2")) then
                  for i = 1, COMPASS.container:GetNumCenterOveredPins() do
                     local _, _, distance, _, _, isSupressed = COMPASS.container:GetCenterOveredPinInfo(i)
                     if not isSupressed then
                        if newDistance == nil or distance < newDistance then
                           newDistance = distance
                        end
                     end
                  end
               end

               if newDistance then
                  local scale = COMPASS_DISTANCE:GetScale()
                     if scale ~= 0 then
                        COMPASS_DISTANCE:SetText(("%d m"):format(zo_round(newDistance * scale)))
                     else
                        COMPASS_DISTANCE:SetText(("%d"):format(zo_round(newDistance * 1000)))
                     end
               else
                  COMPASS_DISTANCE:SetText("")
               end
            end
         end)

      EVENT_MANAGER:UnregisterForEvent("CompassDistance", event)
   end)
